/*
 * Prepare JSON Report
 *
 *     This small program collects test data and global settings to prepare
 * a JSON report for Gradescope autograding.  The following expectations are
 * made:
 *
 *   -  Global settings are kept in settings.json.  Change these for
 *      each assignment.
 *  
 *   -  The following are prepared and sent in by the autograde script:
 *
 *   --  The first parameter is the destination filename:
 *       /autograder/results/results.json
 *
 *   --  The second parameter is the total test run time (as text).
 * 
 *   --  All remaining parameters are the names of files that contain
 *       output information in this form, with data fields being optional in
 *       any order:
 *
 *       :::START [testscriptname]
 *       :::NAME [test_name]
 *       :::NUMBER [test_number]
 *       :::SCORE [points]
 *       :::VISIBILITY [visibility, one of hidden, after_due_date, 
 *                                      after_published, or visible]
 *       [lines of output for this test]
 *       :::END [shell error code for this test]
 *    
 *       (Any line between START and END not recognized is added to 
 *        the output.)
 *
 * This program requires jsoncpp.
 *
 * Peter Jensen
 * February 24, 2019
 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <jsoncpp/json/json.h>
#include <cmath>

using namespace std;

/* Forward declarations */

Json::Value parse_results_file(ifstream & in);

/*
 * Application entry point
 */
int main (int argc, char** argv)
{
  // Assume the first parameter is the output filename.
  // Make sure we have an output filename and execution time.

  if (argc < 3)
  {
    cout << argv[0] << ": Missing parameters - output filename and/or execution time" << endl;
  }

  // Pull in the global settings for these tests.

  Json::Value test_settings;

  ifstream tss("settings.json");
  tss >> test_settings;
  tss.close();

  // Attempt to build a json object, start keeping a score.

  Json::Value results;

  double score = 0.0;
  double max_score = test_settings["max_score"].asDouble();

  // Put in global settings (except for max_score)

  results["execution_time"] = argv[2];

  test_settings.removeMember("max_score");
  
  Json::Value::Members m = test_settings.getMemberNames();

  for (int i = 0; i < m.size(); i++)
    results[m[i]] = test_settings[m[i]];

  // Loop through the output files.  
  //  Extract specific lines to output into the test results.

  results["tests"] = Json::arrayValue;

  for (int i = 3; i < argc; i++)
  {
    ifstream rss(argv[i]);    
    results["tests"][i-3] = parse_results_file(rss);  
    score += results["tests"][i-3]["score"].asDouble();
    rss.close();
  }

  // Insert the score rounded to the nearest int.
  
  // results["score"] = round(score);

  // Prepare the JSON output
  
  Json::FastWriter fastWriter;
  string output = fastWriter.write(results);
  
  // Write it to the required file.
  
  ofstream out(argv[1]);

  out << output << endl;

  out.close();

  // Done.

  return 0;
}

/*
 * Just a simple string trim function.
 */

string trim (string s)
{
  int start = 0;
  while (start < s.size() && s[start] < 33)
    start++;

  int end = s.size();
  while (end > start && s[end-1] < 33)
    end--;

  return s.substr(start, end-start);
}


/*
 * Parse a results file, return a JSON test result 
 */

Json::Value parse_results_file(ifstream & in)
{
  // Prepare test result.

  Json::Value test;

  // Buffer for collecting output.

  string output;

  // Go until no more lines are available or end is reached.

  while (true)
  {
    // Get a line.

    string line;
    getline(in, line);

    if (in.fail()) 
      break;

    istringstream is(line);
    string prefix;
    is >> prefix;

    if (prefix == ":::START")
      continue;
    
    if (prefix == ":::NUMBER")
    {
      string number;
      getline(is, number);
      test["number"] = trim(number);
      continue;
    }

    if (prefix == ":::NAME")
    {
      string name;
      getline(is, name);
      test["name"] = trim(name);
      continue;
    }

    if (prefix == ":::VISIBILITY")
    {
      string visibility;
      getline(is, visibility);
      test["visibility"] = trim(visibility);
      continue;
    }

    if (prefix == ":::SCORE")
    {
      double score;
      is >> score;
      test["score"] = score;
      continue;
    }

    if (prefix == ":::MAX_SCORE")
    {
      double max_score;
      is >> max_score;
      test["max_score"] = max_score;
      continue;
    }

    if (prefix == ":::END")
      break;

    if (output.size() > 0)
      output.append("\n");
    output.append(trim(line));
  }

  // Put in the output.

  test["output"] = output;

  // Return the JSON value for this test.

  return test;
}
